--------------------------------------------------------------------------------
-- Weapon Long-range Homing Missile Launcher + Projectile Missile
-- based on Homing Missile by DC
-- version 1.0
--------------------------------------------------------------------------------

-- Setup Tables
if kifferopa==nil then kifferopa={} end
kifferopa.ihoming={}
kifferopa.ihoming.missile={}
kifferopa.ihoming.settings={}
--[[ SETTINGS
	set your options here
]]
-- maxtargets: maximum amount of checkpoints the player can select (must be at least 1)
kifferopa.ihoming.settings.maxtargets=4
-- inittime: time in frames needed for the rocket to start the homing process (50 frames = 1 second)
kifferopa.ihoming.settings.inittime=75
-- maximum amount of frames the rocket can be propelled, divided by maxtargets. Total frame time is fuelpertarget*maxtargets
kifferopa.ihoming.settings.fuelpertarget=300
-- end of settings

-- Load & Prepare Ressources
kifferopa.ihoming.gfx_wpn=loadgfx("weapons/launcher.bmp")					-- Weapon Image
setmidhandle(kifferopa.ihoming.gfx_wpn)
kifferopa.ihoming.gfx_pro=loadgfx("weapons/improvedhomingmissile.bmp")				-- Projectile Image
setmidhandle(kifferopa.ihoming.gfx_pro)
kifferopa.ihoming.sfx_attack=loadsfx("rocketrelease.wav")					-- Attack Sound
kifferopa.ihoming.sfx_ihoming=loadsfx("homing.wav")						-- Homing Sound
kifferopa.ihoming.gfx_x=loadgfx("icon_x.png")							-- X (marks the spot)
setmidhandle(kifferopa.ihoming.gfx_x)

--------------------------------------------------------------------------------
-- Weapon: Homing Missile Launcher
--------------------------------------------------------------------------------

kifferopa.ihoming.id=addweapon("kifferopa.ihoming","Long-range Homing Missile",kifferopa.ihoming.gfx_pro,0,2)

kifferopa.ihoming.targets={}
kifferopa.ihoming.targetindex=0
kifferopa.ihoming.lasttargetindex=-1

function kifferopa.ihoming.draw()								-- Draw
	setblend(blend_alpha)
	setalpha(1)
	setcolor(51,89,199)
	drawinhand(kifferopa.ihoming.gfx_wpn,6,0)
	-- HUD chargebar
	if weapon_charge>0 and weapon_shots==0 then
		hudchargebar(weapon_charge,100)
	end
	if(weapon_mode==0) then
		kifferopa.ihoming.targets={}
		kifferopa.ihoming.targetindex=0
		kifferopa.ihoming.lasttargetindex=-1
	end

	-- mark all spots
	setscale(1,1)
	setrotation(0)
	setalpha(1)
	for i=0,kifferopa.ihoming.lasttargetindex do
		-- cleared targets become green
		if(kifferopa.ihoming.targets[i].cleared==true) then
			setcolor(0,255,0)
		-- and targeting
		elseif(i==kifferopa.ihoming.targetindex and weapon_shots==1) then
			setcolor(255,255,255)
		-- other targets get a color fading from red to yellow (target zero is red, last target is yellow,
		-- targets "between" get an orange color
		else
			green=math.ceil(i*(256/kifferopa.ihoming.settings.maxtargets))
			setcolor(255,green,0)
		end
		drawimage(kifferopa.ihoming.gfx_x,kifferopa.ihoming.targets[i].x,kifferopa.ihoming.targets[i].y)
	end

	-- HUD Crosshair + Positioning
	if weapon_shots==0 then
		hudammobar(kifferopa.ihoming.settings.maxtargets-weapon_mode,kifferopa.ihoming.settings.maxtargets)
		-- if the player has marked at least one point on the map
		if(weapon_mode>=1) then
			-- show crosshair
			hudcrosshair(6,3)
		end
		if(weapon_mode<kifferopa.ihoming.settings.maxtargets) then
			-- let the player click
			-- note that the position icons actually WILL be shown via this draw function
			hudpositioning(pos_invisible)
		end
		if(weapon_position==1 and weapon_mode<kifferopa.ihoming.settings.maxtargets) then
			kifferopa.ihoming.targets[weapon_mode]={x=weapon_x,y=weapon_y,cleared=false}
			kifferopa.ihoming.lasttargetindex=kifferopa.ihoming.lasttargetindex+1
			weapon_mode=weapon_mode+1
			weapon_position=0
		end
	end
end

function kifferopa.ihoming.attack(attack)									-- Attack
	if (weapon_shots<=0) and (weapon_mode>=1) then
		-- Charge
		if (attack==1) then
			weapon_charge=weapon_charge+1							-- Increase charge
		end
		-- Fire a projectile (on release/full charge)
		if (attack==0 and weapon_charge>0) or (weapon_charge>=100) then
			-- No more weapon switching!
			useweapon(0)
			playsound(kifferopa.ihoming.sfx_attack)
			weapon_shots=weapon_shots+1
			kifferopa.ihoming.targetindex=-1
			id=createprojectile(kifferopa.ihoming.missile.id)
			projectiles[id]={}
			-- Ignore collision with current player at beginning
			projectiles[id].ignore=playercurrent()
			-- Set initial position of projectile
			projectiles[id].x=getplayerx(0)+(6*getplayerdirection(0))+math.sin(math.rad(getplayerrotation(0)))*10.0
			projectiles[id].y=getplayery(0)+3-math.cos(math.rad(getplayerrotation(0)))*10.0
			-- Set Timer (time until homing, 1.5 s)
			projectiles[id].timer=kifferopa.ihoming.settings.inittime
			-- set fuel
			projectiles[id].fuel=kifferopa.ihoming.settings.fuelpertarget*kifferopa.ihoming.settings.maxtargets
			-- Initial movement
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*22
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*22
			projectiles[id].x=projectiles[id].x-projectiles[id].sx
			projectiles[id].y=projectiles[id].y-projectiles[id].sy
			kifferopa.ihoming.missile.move(id,0)
			-- Set speed of projectile
			projectiles[id].sx=math.sin(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			projectiles[id].sy=-math.cos(math.rad(getplayerrotation(0)))*(weapon_charge/100.0)*10.0
			-- Effects
			recoil(2)
			-- End Turn
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile: Missile
--------------------------------------------------------------------------------

kifferopa.ihoming.missile.id=addprojectile("kifferopa.ihoming.missile")		-- Add Projectile

function kifferopa.ihoming.missile.draw(id)							-- Draw
	-- Setup draw mode
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	setscale(1,1)
	-- Calculate projectile rotation
	setrotation(math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy)))
	-- Draw projectile
	drawimage(kifferopa.ihoming.gfx_pro,projectiles[id].x,projectiles[id].y)
	-- Draw Arrow if out of Screen
	outofscreenarrow(projectiles[id].x,projectiles[id].y)
end

function kifferopa.ihoming.missile.update(id)						-- Update
	kifferopa.ihoming.missile.move(id,1)
end

function kifferopa.ihoming.missile.move(id,fx)
	rot=math.deg(math.atan2(projectiles[id].sx,-projectiles[id].sy))
	-- Particle Tail
	if (fx==1) then
		particle(p_smoke,projectiles[id].x-math.sin(math.rad(rot))*7,projectiles[id].y+math.cos(math.rad(rot))*7)
		particlespeed(math.random(-2,2)*0.1,math.random(-2,2)*0.1)
		particlefadealpha(0.01)
		if projectiles[id].timer<0 and projectiles[id].timer>-250 then
			particle(p_lightpuff,projectiles[id].x-math.sin(math.rad(rot))*6,projectiles[id].y+math.cos(math.rad(rot))*6)
			particlefadealpha(0.04)
		end
	end
	-- Wind + Gravity influence on speed
	projectiles[id].sx=projectiles[id].sx+getwind()
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- reduce fuel
	projectiles[id].fuel=projectiles[id].fuel-1
	-- Homing
	explode=0
	if (fx==1) then
		projectiles[id].timer=projectiles[id].timer-1
	end
	if projectiles[id].timer<=0 and projectiles[id].timer>-250 and kifferopa.ihoming.targetindex<=kifferopa.ihoming.lasttargetindex then
		if(kifferopa.ihoming.targetindex==-1) then
			kifferopa.ihoming.targetindex=0
		end

		if(kifferopa.ihoming.targetindex<kifferopa.ihoming.lasttargetindex) then
			if kifferopa.ihoming.missile.distancetotarget(projectiles[id])<=30 then
				-- target cleared! fly to next one.
				kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].cleared=true
				projectiles[id].timer=1
				kifferopa.ihoming.targetindex=kifferopa.ihoming.targetindex+1
			end
		end

		-- Start Homing FX
		if projectiles[id].timer==0 then
			playsound(kifferopa.ihoming.sfx_ihoming)
			particle(p_ring,projectiles[id].x,projectiles[id].y)
			particlecolor(51,99,199)
		end
		-- Homing Calculation
		aimangle=math.deg(math.atan2(kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].x-projectiles[id].x,-(kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].y-projectiles[id].y)))
		speed=math.sqrt(math.pow(projectiles[id].sx,2)+math.pow(projectiles[id].sy,2))
		if speed<8.0 then
			speed=speed+0.15
			if speed>8.0 then speed=8.0 end
		end
		rot=rot+angledelta(rot,aimangle)*0.08
		projectiles[id].sx=math.sin(math.rad(rot))*speed
		projectiles[id].sy=-math.cos(math.rad(rot))*speed	
		-- Fuel empty
		if projectiles[id].fuel<=0 then 
			explode=1
		end
	
	end
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy))/3)
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		projectiles[id].x=projectiles[id].x+msubx
		projectiles[id].y=projectiles[id].y+msuby
		-- Collision
		if collision(col3x3,projectiles[id].x+math.sin(math.rad(rot))*3,projectiles[id].y-math.cos(math.rad(rot))*3)==1 or explode==1 then
			if terraincollision()==1 or objectcollision()>0 or playercollision()~=projectiles[id].ignore or explode==1 then
				-- Cause damage
				arealdamage(projectiles[id].x,projectiles[id].y,100,40)
				-- Destroy terrain
				terrainexplosion(projectiles[id].x,projectiles[id].y,25,1)
				-- Crater
				grey=math.random(0,40)
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_crater100,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				else
					terrainalphaimage(gfx_crater125,projectiles[id].x,projectiles[id].y,math.random(6,9)*0.1,grey,grey,grey)
				end
				-- mark last target as cleared (if it was targeted and missile was close enough to it)
				if(kifferopa.ihoming.targetindex==kifferopa.ihoming.lasttargetindex) then
					if kifferopa.ihoming.missile.distancetotarget(projectiles[id])<30 then
						kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].cleared=true
					end
				end
				-- Free projectile
				freeprojectile(id)
				break
			end
		else
			projectiles[id].ignore=0
		end
		-- Water
		if (projectiles[id].y)>getwatery()+5 then
			-- Effects
			particle(p_waterhit,projectiles[id].x,projectiles[id].y)
			playsound(sfx_hitwater1)
			-- Free projectile
			freeprojectile(id)
			break
		end
	end
	-- Scroll to projectile
	scroll(projectiles[id].x,projectiles[id].y)
end

function kifferopa.ihoming.missile.distancetotarget(projectile)
	return math.sqrt(math.pow(projectile.x-kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].x,2)+math.pow(projectile.y-kifferopa.ihoming.targets[kifferopa.ihoming.targetindex].y,2))
end
